#include<fstream.h>
#include"dialdevice_impl.h"
#include"dialupmanager_erg.h"
#include"corbalogger.h"
extern CorbaLog corbalog;
int dialdevice_debuglevel=0;

// Implementation for interface DialDevice
DialDevice_impl::DialDevice_impl(const char *pname,const char *pdesc,IsdnSystem *isdnsys):Device_impl(pname,pdesc),POA_DialupManager::DialDevice()
{
    is=isdnsys;
    altstatus   = is->getstatus(iname);
    altdialmode = is->getdialmode(iname);
}

void
DialDevice_impl::dial()
  throw(
    ::CORBA::SystemException,
    DialupManager::Syserr)

{
    is->dial(iname);
}


void
DialDevice_impl::hangup()
  throw(
    ::CORBA::SystemException,
    DialupManager::Syserr)

{
    is->hangup(iname);
    // hangup heit hangup
    if(pinger.status() == Process::running)
    {
	stop_pinger();
    }
    altdialmode = getdialmode();
}


void
DialDevice_impl::setdialmode( DialupManager::DialDevice::DialMode d )
  throw(
    ::CORBA::SystemException,
    DialupManager::Syserr)

{
  DialupManager::DialDevice::DialMode retval;
  is->setdialmode(iname,d);
  if(altdialmode != DialupManager::DialDevice::permanent
     && d == DialupManager::DialDevice::permanent)
  {
      // terminiere alten pinger 
      if(pinger.status()==Process::running)
      {
	  stop_pinger();
	
      }
      /*------ start pinger ---------------*/
      const char *arg[5];
      int a=0;
      arg[a++]="ping";
      arg[a++]="-i3";
      arg[a++]=is->getpeerip(iname);
      arg[a]=NULL;

      pinger.ssys("/bin/ping",arg);
      corbalog.log(Syslog::Info,"%s : ping %s %s ",(char *)iname,arg[1],arg[2]);
  }
  if(altdialmode == DialupManager::DialDevice::permanent
     && d != DialupManager::DialDevice::permanent)
  {
      /*------ stop pinger ----------------*/
      if(pinger.terminate(SIGINT)>=0)
      {
	  pinger.wait();
	  corbalog.log(Syslog::Info,"%s : pinger stopped",(char *)iname);
      }
      else
      {
	  corbalog.log(Syslog::Warning,"%s stop pinger: %m",(char *)iname);
      }
  }
  aktionifchanged();
}


DialupManager::DialDevice::DialMode
DialDevice_impl::getdialmode()
  throw(
    ::CORBA::SystemException,
    DialupManager::Syserr)

{
    DialupManager::DialDevice::DialMode emp = is->getdialmode(iname);
    if(pinger.status()==Process::running && emp != DialupManager::DialDevice::off)
    {
	return DialupManager::DialDevice::permanent;
    }
    else
    {
	return emp;
    }
    
    // return is->getdialmode(iname); // was bei permanent ?
    // return altdialmode; // was aber bei nderungen mit isdnctrl ?
}


DialupManager::DialDevice::Status
DialDevice_impl::getstatus()
  throw(
    ::CORBA::SystemException,
    DialupManager::Syserr)

{
    return is->getstatus(iname);
}


CORBA::Long
DialDevice_impl::activechannels()
  throw(
    ::CORBA::SystemException,
    DialupManager::Syserr)

{
  CORBA::Long retval;
  retval=1;
  return retval; 
}

void
DialDevice_impl::activate()
  throw(
    ::CORBA::SystemException,
    DialupManager::Syserr)

{
    setdialmode(DialupManager::DialDevice::automatic);
    dial();
}


void
DialDevice_impl::deactivate()
  throw(
    ::CORBA::SystemException,
    DialupManager::Syserr)

{
    setdialmode(DialupManager::DialDevice::manual);
    hangup();
}

char *
DialDevice_impl::get_our_ip()
  throw(
    ::CORBA::SystemException,
    ::DialupManager::Syserr)

{
  char* retval;

  retval = CORBA::string_dup(is->getip(iname));
  return retval; 
}


char*
DialDevice_impl::get_peer_ip()
  throw(
    ::CORBA::SystemException,
    ::DialupManager::Syserr)

{
  char* retval;

  retval = CORBA::string_dup(is->getpeerip(iname));
  return retval; 
}

CosEventChannelAdmin::EventChannel_ptr
DialDevice_impl::geteventchannel()
  throw(
    ::CORBA::SystemException,
    DialupManager::Syserr)

{
  return CosEventChannelAdmin::EventChannel::_duplicate(eventchannel);
}

/*------------ Push-Supplier -----------------------------*/
void
DialDevice_impl::disconnect_push_supplier()
  throw(
    ::CORBA::SystemException)

{
    if(dialdevice_debuglevel > 0)
    {
	corbalog << "Pushsupplier von " << iname << " wird abgehngt " << endl;
    }
    eventconsumer = CosEventChannelAdmin::ProxyPushConsumer::_nil();
}

/*------------ Nicht-Schnittstellenoperationene ----------*/
extern CORBA::ORB_var orb;
void DialDevice_impl::initpushsupplier(CosEventComm::PushSupplier_ptr psup,
				       CosEventChannelAdmin::EventChannel_ptr ech)
{
    CosEventChannelAdmin::SupplierAdmin_var chadmin;
    psupplier=psup;
    eventchannel=ech;

    if(dialdevice_debuglevel > 0)
    {
	corbalog << "Hole mir einen Proxy-Consumer ... " << endl;
    }
    chadmin = eventchannel->for_suppliers();
    eventconsumer= chadmin->obtain_push_consumer();
    eventconsumer->connect_push_supplier(psupplier);
    
    if(dialdevice_debuglevel > 1)
    {
	corbalog << "eventchannel: "
		 << orb->object_to_string(eventchannel) << endl;
	corbalog << "eventconsumer: "
		 << orb->object_to_string(eventconsumer) << endl;
	corbalog << "pushsupplier:  "  
		 << orb->object_to_string(psupplier) << endl;
    }
}

void DialDevice_impl::aktionifchanged()
{
    CORBA::Any any;
    DialupManager::dmstate dms;

    try
    {
	dms.status = getstatus();
	dms.dm     = getdialmode();

	if(altstatus != dms.status)
	{
	    corbalog << iname << " Aktion " << dms.status << endl;
	}
	if(altstatus != dms.status || altdialmode != dms.dm)
	{
	    any <<= dms;
	    if(!CORBA::is_nil(eventconsumer))
	    {
		eventconsumer->push(any);
	    }
	    else
	    {
		if(dialdevice_debuglevel > 0)
		    corbalog << "eventconsumer leider abgehngt ... " << endl;
	    }
	}
	altstatus = dms.status;
	altdialmode = dms.dm;
    }
    catch(...)
    {
	corbalog << iname << ".aktionifchanged(): Fehler aufgetreten " << endl;
    }
}

void DialDevice_impl::stop_pinger()
{
    if(pinger.terminate(SIGINT)>=0)
    {
	pinger.wait();
	corbalog.log(Syslog::Info,"%s : pinger stopped",(char *)iname);
    }
    else
    {
	corbalog.log(Syslog::Warning,"%s stop pinger: %m",(char *)iname);
    }
}

extern CORBA::ORB_var orb;
void DialDevice_impl::store(const char *datei)
{
    if(dialdevice_debuglevel > 0)
	corbalog << "Speichere in " << datei << endl;
    CORBA::String_var evref = orb->object_to_string(eventchannel);
 
    ofstream speich(datei);
 
    speich << idesc << endl;
    speich << evref << endl;
    speich.close();
}

void DialDevice_impl::cleanup()
{
    if(dialdevice_debuglevel > 0)
	corbalog << "Disconnect Push-Consumer ... " << endl;
    if(CORBA::is_nil(eventconsumer))
    {
	if(dialdevice_debuglevel > 0)
	    corbalog << "... ist schon lange !" << endl;
    }
    else
    {
	if(dialdevice_debuglevel > 0)
	    corbalog << "eventconsumer: " << orb->object_to_string(eventconsumer) << endl;
	eventconsumer->disconnect_push_consumer();
	eventconsumer = CosEventChannelAdmin::ProxyPushConsumer::_nil();
	if(dialdevice_debuglevel > 0)
	    corbalog << "gemacht " << endl;
    }
}

