#include<sys/types.h>
#include<fcntl.h>
#include<unistd.h>
#include<errno.h>
#include<sys/ioctl.h>
#include<linux/isdn.h>
#include<sys/ioctl.h>
#include<netinet/in.h>
#include<arpa/inet.h>
#include<sys/socket.h>
#include<linux/if.h>
#include<linux/sockios.h>
#include"isdnsystem.h"
#include"rwstrtok.h"
#include"corbalogger.h"

extern CorbaLog corbalog;
int isdnsystem_debuglevel = 0;
/*--------------------- internetsystem ----------------------*/
InternetSystem::InternetSystem()
{
    sockfd=socket(AF_INET,SOCK_DGRAM,0);
    if(sockfd<0)
    {
	char msg[256];
	snprintf(msg,256,"Kann keinen socket(AF_INET,SOCK_DGRAM) oeffnen: %s",
		 strerror(errno));
	mico_throw(DialupManager::Syserr(errno,msg));
    }
}

InternetSystem::~InternetSystem()
{
    close(sockfd);
}


const char *InternetSystem::getip(const char *in)
{
    struct ifreq req;
    strncpy(req.ifr_name,in,sizeof(req.ifr_name));

    if(ioctl(sockfd,SIOCGIFADDR,&req)<0)
    {
	char msg[256];
	snprintf(msg,256,"ioctrl(%s,,SIOCGIFADDR): %s",strerror(errno));
	mico_throw(DialupManager::Syserr(errno,msg));
    }
    
    return (const char *)inet_ntoa(((struct sockaddr_in *)&(req.ifr_addr))->sin_addr);

}

const char *InternetSystem::getpeerip(const char *in)
{
    struct ifreq req;
    strncpy(req.ifr_name,in,sizeof(req.ifr_name));

    if(ioctl(sockfd,SIOCGIFDSTADDR,&req)<0)
    {
	char msg[256];
	snprintf(msg,256,"ioctrl(%s,SIOCGIFDSTADDR): %s",strerror(errno));
	mico_throw(DialupManager::Syserr(errno,msg));
    }
    return (const char *)inet_ntoa(((struct sockaddr_in *)&(req.ifr_addr))->sin_addr);
}

/*--------------------- isdn-system -------------------------*/
IsdnSystem::IsdnSystem()
{
    ictrlfd=open("/dev/isdnctrl",O_RDONLY| O_NONBLOCK);
    if(ictrlfd < 0)
    {
	char msg[256];
	snprintf(msg,256,"Kann /dev/isdnctrl nicht oeffnen: %s",
		 strerror(errno));
	mico_throw(DialupManager::Syserr(errno,msg));
    }
    ictrl2fd = -1;
    if(0)
    {
	ictrl2fd=open("/dev/isdnctrl2",O_RDONLY| O_NONBLOCK);
	if(ictrl2fd < 0)
	{
	    char msg[256];
	    snprintf(msg,256,"Kann /dev/isdnctrl2 nicht oeffnen: %s",
		     strerror(errno));
	    mico_throw(DialupManager::Syserr(errno,msg));
	}
    }
    iinfofd=open("/dev/isdninfo",O_RDONLY| O_NONBLOCK);
    if(iinfofd < 0)
    {
	char msg[256];
	snprintf(msg,256,"Kann /dev/isdninfo nicht oeffnen: %s",
		 strerror(errno));
	mico_throw(DialupManager::Syserr(errno,msg));
    }
#ifndef NO_ISDNCTI
    ictifd=open("/dev/isdncti",O_RDONLY| O_NONBLOCK);
    if(ictifd < 0)
    {
	char msg[256];
	snprintf(msg,256,"Kann /dev/isdncti nicht oeffnen: %s",
		 strerror(errno));
	mico_throw(DialupManager::Syserr(errno,msg));
    }
#else
    ictifd = -1;
#endif
}

IsdnSystem::~IsdnSystem()
{
    if(ictrlfd > 0)  close(ictrlfd);
    if(ictrl2fd > 0) close(ictrl2fd);
    close(iinfofd);
    if(ictifd > 0) close(ictifd);
}


void IsdnSystem::dial(const char *id)
	throw(DialupManager::Syserr)
{
    int r;
    if((r=ioctl(ictrlfd,IIOCNETDIL,id))<0)
    {
	char msg[256];
	snprintf(msg,256,"dial %s: %s",id,strerror(errno));
	corbalog.log(Syslog::Err,msg);
	mico_throw(DialupManager::Syserr(errno,msg));
    }
    corbalog.log(Syslog::Info,"%s waehlt ... ",id);
}

void IsdnSystem::hangup(const char *id)
	throw(DialupManager::Syserr)
{
    int r;
    if((r=ioctl(ictrlfd,IIOCNETHUP,id))<0)
    {
	char msg[256];
	snprintf(msg,256,"hangup %s: %s",id,strerror(errno));
	corbalog.log(Syslog::Err,msg);
	mico_throw(DialupManager::Syserr(errno,msg));
    }
    corbalog.log(Syslog::Info,"Hangup: %s %s",id,( r ? " war nicht verbunden" : " aufgelegt"));
}

void IsdnSystem::setdialmode(const char *id,DialupManager::DialDevice::DialMode d )
	throw(DialupManager::Syserr)
{
  DialupManager::DialDevice::DialMode retval;
  isdn_net_ioctl_cfg icfg;

  strncpy(icfg.name,id,9);icfg.name[9]='\0';
  if(ioctl(ictrlfd,IIOCNETGCF,&icfg)< 0)
  {
      char msg[256];
      snprintf(msg,256,"ioctl(/dev/isdnctrl,IIOCNETGCF) : %s",
	       strerror(errno));
      corbalog.log(Syslog::Err,msg);
      mico_throw(DialupManager::Syserr(errno,msg));
  }
  switch(d)
  {
  case DialupManager::DialDevice::off :
      icfg.dialmode=ISDN_NET_DM_OFF;
      break;
  case DialupManager::DialDevice::manual:
      icfg.dialmode=ISDN_NET_DM_MANUAL;
      break;
  case DialupManager::DialDevice::automatic:
  case DialupManager::DialDevice::permanent:
      icfg.dialmode=ISDN_NET_DM_AUTO;
      break;
  default:
      mico_throw(DialupManager::Syserr(-1,"setdialmode: Illegal Dialmode"));
  }
  if(ioctl(ictrlfd,IIOCNETSCF,&icfg)< 0)
  {
      char msg[256];
      snprintf(msg,256,"ioctl(/dev/isdnctrl,IIOCNETGCF) : %s",
	       strerror(errno));
      corbalog.log(Syslog::Err,msg);
      mico_throw(DialupManager::Syserr(errno,msg));
  }
}

DialupManager::DialDevice::DialMode IsdnSystem::getdialmode(const char *id)
	throw(DialupManager::Syserr)
{
  DialupManager::DialDevice::DialMode retval;
  isdn_net_ioctl_cfg icfg;

  strncpy(icfg.name,id,9);icfg.name[9]='\0';
  if(ioctl(ictrlfd,IIOCNETGCF,&icfg)< 0)
  {
      char msg[256];
      snprintf(msg,256,"ioctl(/dev/isdnctrl,IIOCNETGCF) : %s",
	       strerror(errno));
      corbalog.log(Syslog::Err,msg);
      mico_throw(DialupManager::Syserr(errno,msg));
  }
  switch(icfg.dialmode)
  {
  case ISDN_NET_DM_OFF:    return DialupManager::DialDevice::off;
  case ISDN_NET_DM_MANUAL: return DialupManager::DialDevice::manual;
  case ISDN_NET_DM_AUTO:   return DialupManager::DialDevice::automatic;
  default:
      mico_throw(DialupManager::Syserr(-1,"Illegal Dialmode"));
  }
  return retval; 
}
    
DialupManager::DialDevice::Status IsdnSystem::getstatus(const char *iname)
	throw(DialupManager::Syserr)
{
  DialupManager::DialDevice::Status retval;
  isdn_net_ioctl_phone phone;
  int r;

  memset(&phone, 0, sizeof phone);
  strncpy(phone.name, iname, sizeof phone.name);
  r=ioctl(iinfofd,IIOCNETGPN, &phone);

  if(r==0)
  {
      retval = (phone.outgoing ? DialupManager::DialDevice::outgoing
		               : DialupManager::DialDevice::incoming);
  }
  else if(errno == ENOTCONN) 
  {
      retval = DialupManager::DialDevice::offline;
  }
  else
  {
      char msg[256];
      snprintf(msg,256,"ioctl(/dev/isdninfo,IIOCNETGPN) : %s",
	       strerror(errno));
      corbalog.log(Syslog::Err,msg);
      mico_throw(DialupManager::Syserr(errno,msg));
  }
  return retval;
}

/*------------ Isdninfo ----------------------------*/
Isdninfo::Isdninfo(char *_name)
{ 
    strncpy(name,_name,16);name[15]='\0';
    ch=0;dr=0;usage=0;flags=0;phone[0]='\n';
    nxt=NULL;
}

int Isdninfo::bound()
{
    return (ch>=0 && dr>=0);
}
int Isdninfo::exclusive()
{
    return (usage & ISDN_USAGE_EXCLUSIVE);
}

int Isdninfo::outgoing()
{
    return (usage & ISDN_USAGE_OUTGOING);
}

const char *Isdninfo::alswas()
{
    switch(usage & ISDN_USAGE_MASK)
    {
    case ISDN_USAGE_RAW:   return "raw";
    case ISDN_USAGE_MODEM: return "Modem";
    case ISDN_USAGE_NET:   return "Net";
    case ISDN_USAGE_VOICE: return "Voice";
    case ISDN_USAGE_FAX:   return "Fax";
    default: return "idle";
    }
}

int interpretinfo(Isdninfo *&wurzel,char* puffer)
{
    char *zeile;
    
    Strtok lb(puffer,"\n");
    zeile=lb(); if(!zeile || strncmp(zeile,"idmap:",6)!=0) { return -1;; }

    if(!wurzel) // sonst ist das zeugs schon allokiert
    {
	char *cn;
	Strtok stk(zeile," \t-");
	Isdninfo **a = &wurzel;
	for(stk.next();cn=stk();stk.next())
	{
	    *a = new Isdninfo(cn);
	    a = &((*a)->nxt);
	}
    }
    
    lb.next();zeile=lb();
    if(!zeile || strncmp(zeile,"chmap:",6)!=0) { return -1;; }

    {
	int i;
	char *ai;
	Strtok stk(zeile," \t");stk.next();
	for(Isdninfo *akt=wurzel;(ai=stk()) && akt;stk.next(), akt=akt->nxt)
	{
	    akt->ch = atoi(ai);
	}
    }
    lb.next();zeile=lb();
    if(!zeile || strncmp(zeile,"drmap:",6)!=0) { return -1;; }

    {
	int i;
	char *ai;
	Strtok stk(zeile," \t");stk.next();
	for(Isdninfo *akt=wurzel;(ai=stk()) && akt;stk.next(), akt=akt->nxt)
	{
	    akt->dr = atoi(ai);
	}
    }

    lb.next(); zeile=lb();
    if(!zeile || strncmp(zeile,"usage:",6)!=0) { return -1;; }
    
    {
	int i;
	char *ai;
	Strtok stk(zeile," \t");stk.next();
	for(Isdninfo *akt=wurzel;(ai=stk()) && akt;stk.next(), akt=akt->nxt)
	{
	    akt->usage = atoi(ai);
	}
    }
    lb.next();zeile=lb();
    if(!zeile || strncmp(zeile,"flags:",6)!=0) { return -1;; }
    
    {
	int i;
	char *ai;
	Strtok stk(zeile," \t-");stk.next();
	for(Isdninfo *akt=wurzel;(ai=stk()) && akt;stk.next(), akt=akt->nxt)
	{
	    akt->flags = atoi(ai);
	}
    }
    lb.next();zeile=lb(); if(!zeile || strncmp(zeile,"phone:",6)!=0) { return -1;; }

    {
	int i;
	char *ai;
	Strtok stk(zeile," \t-");stk.next();
	for(Isdninfo *akt=wurzel;(ai=stk()) && akt;stk.next(), akt=akt->nxt)
	{
	    strncpy(akt->phone,ai,32);akt->phone[31]='\0';
	}
    }
    return 0;
}

void printisdninfo(ostream &str,Isdninfo *wurzel)
{
    for(Isdninfo *a=wurzel;a;a=a->nxt)
    {
	str  << a->name << " " << a->alswas() << " "
	     << (a->outgoing() ? "out" : "in") 
	     << (a->exclusive() ? "excl" : "")
	     << " flags=" << a->flags
	     << " phone=" << a->phone << endl;
    }
}
