/***************************************************************************
 * Header...: isdnsystem.h                                                 *
 ***************************************************************************/
#ifndef ISDNSYSTEM_H
#define ISDNSYSTEM_H
#include"dialupmanager.h" // DialupManager::Syserr

/* IsdnSystem ist eine Abstraktion des Isdn4Linux-Systems 
   bzw. kann eine werden.
   ggf. kann diese Klasse durch einen Simulator ersetzt werden,
   daher sind die Operationen wie hangup und dergl hier drin.

   Das InternetSystem enthaelt Funktionen zum ermitteln von
   Informationen bzgl. eigener Ip-Adresse und die des gegenueber
*/

class InternetSystem
{
    int sockfd;
 public:
    InternetSystem();
    ~InternetSystem();
    const char *getip(const char *in);
    const char *getpeerip(const char *in);
};

class IsdnSystem : public InternetSystem
{
    int ictrlfd;
    int ictrl2fd; // Falls zweite Beobachtungskarte vorhanden
    int iinfofd;
    int ictifd;
public:
    IsdnSystem();
    ~IsdnSystem();
    int ctrlfd()  { return ictrlfd; }
    int ctrl2fd() { return ictrl2fd; } // Falls zweite Beobachtungskarte
    int infofd()  { return iinfofd; }
    int ctifd()   { return ictifd; }
    void dial(const char *id)
	throw(DialupManager::Syserr);
    void hangup(const char *id)
	throw(DialupManager::Syserr);
    void setdialmode(const char *id,DialupManager::DialDevice::DialMode d )
	throw(DialupManager::Syserr);

    DialupManager::DialDevice::DialMode getdialmode(const char *id)
	throw(DialupManager::Syserr);
    
    DialupManager::DialDevice::Status getstatus(const char *id)
	throw(DialupManager::Syserr);
};

struct Isdninfo
{
    char name[16];
    int ch;
    int dr;
    int usage;
    int flags;
    char phone[32];

    Isdninfo *nxt;

    Isdninfo(char *_name);
    int bound();
    int exclusive();
    int outgoing();
    const char *alswas();
};

int interpretinfo(Isdninfo *&wurzel,char* puffer);
void printisdninfo(ostream& str,Isdninfo *wurzel);
#endif



