#include<iostream.h>
#include<fstream.h>
#include<signal.h>
#include<string.h>
#include<unistd.h>
#include<mico/CosNaming.h>
#include"corbalogger.h"
#include"isdnevent.h"
#ifdef MIT_ISDN_LOG
#include"isdnctrl.h"
#endif
#include"isdncti.h"
#include"dialdevice_impl.h"
#include"control_impl.h"
#include"ifnameit.h"
#include"config.h"

int main_debuglevel=0;
CorbaLog corbalog;
IsdnSystem isdnsystem;
static DialDeviceServantMap ddsvmap;


char *descdatprefix = DATAPATH;
CORBA::ORB_ptr orb;
PortableServer::POA_var rootpoa;
PortableServer::POA_var interfpoa;

SimpleEventChannelAdmin::EventChannelFactory_var eventchannelfactory;
CosEventChannelAdmin::ConsumerAdmin_var cticonsumeradm;

/*------------------- Control-Interface ----------------------------*/
class DmControl_impl: public Control_impl
{
public:
#ifdef LIBSTD
    DmControl_impl::DmControl_impl(const char *pname):Control_impl(pname) {}
    void init_debugtab();
#else
    DmControl_impl::DmControl_impl(const char *pname,Debugvarmap _dvm,int dvmsz):Control_impl(pname,_dvm,dvmsz) {}
#endif
  void exportnames( CosNaming::NamingContext_ptr nc) 
      throw(::CORBA::SystemException);
};


extern int dialdevice_debuglevel;
extern int isdnevent_debuglevel;
extern int isdnsystem_debuglevel;
extern int main_debuglevel;

#ifdef LIBSTD  
void DmControl_impl::init_debugtab()
{
   dvm["dialdevice"] = &dialdevice_debuglevel;
   dvm["isdnevent"] = &isdnevent_debuglevel;
   dvm["isdnsystem"] = &isdnsystem_debuglevel;
   dvm["isdncti"] = &isdncti_debuglevel;
   dvm["main"] = &main_debuglevel;
}
#else
Debugvar dvmt[]=
{
   {"dialdevice",&dialdevice_debuglevel},
   {"isdnevent",&isdnevent_debuglevel},
   {"isdnsystem",&isdnsystem_debuglevel},
   {"isdncti",&isdncti_debuglevel},
   {"main",&main_debuglevel},
   {NULL,NULL}
};
#endif

void
DmControl_impl::exportnames(CosNaming::NamingContext_ptr nc )
  throw(::CORBA::SystemException)

{
    /*---- Verffentliche unsere Interfaces im Nameserver ----*/  
  CosNaming::Name contextname;
  contextname.length (1);
  contextname[0].id = CORBA::string_dup ("DialupManager");
  contextname[0].kind = CORBA::string_dup ("");
  CORBA::Object_var ddmnc_obj;
  CosNaming::NamingContext_var ddmnc;
    
  corbalog << "Verffentliche unsere Interfaces im Nameserver ... " << endl;

  try
  {
      ddmnc_obj = nc->bind_new_context(contextname);
      if(CORBA::is_nil(ddmnc_obj))
      {
	  corbalog.log(Syslog::Err,"Dialupmanager-Kontext ist NIL");
	  return;
      }
      ddmnc=CosNaming::NamingContext::_narrow(ddmnc_obj);
  }
  catch(CosNaming::NamingContext::AlreadyBound_catch &nex)
  {
      corbalog << "Hoppla: Schon gebunden !" << endl;
      ddmnc_obj = nc->resolve(contextname);
      ddmnc=CosNaming::NamingContext::_narrow(ddmnc_obj);
  }
  catch(CosNaming::NamingContext::NotFound_catch &nex)
  {
      corbalog.setprio(Syslog::Err);
      corbalog << nex << ": da " << nex.why << endl;
      corbalog.setprio(Syslog::Debug);
      return;
  }
  catch (CosNaming::NamingContext::CannotProceed_catch &exc)
  {
      corbalog.log(Syslog::Err,"CannotProceed exception");
      return;
  }
  catch (CosNaming::NamingContext::InvalidName_catch &exc)
  {
      corbalog.log(Syslog::Err,"InvalidName exception");
      return;
  }

  /*---- CTI eintragen --------------*/
  try
  {
      CosNaming::Name name;
      name.length(1);
      name[0].id = CORBA::string_dup ("cti");
      name[0].kind = CORBA::string_dup ("");
      ddmnc->rebind(name,cticonsumeradm.in());
  }
  catch(CosNaming::NamingContext::NotFound_catch &nex)
  {
      corbalog.setprio(Syslog::Err);
      corbalog << nex << ": da " << nex.why << endl;
      corbalog.setprio(Syslog::Debug);
  }
  catch (CosNaming::NamingContext::CannotProceed_catch &exc)
  {
      corbalog.log(Syslog::Err,"CannotProceed exception");
  }
  catch (CosNaming::NamingContext::InvalidName_catch &exc)
  {
      corbalog.log(Syslog::Err,"InvalidName exception");
  }
  /*--- Trage die Interfaces ein ----*/
  if(main_debuglevel > 0) corbalog << "Interfaces: " << endl;
  Ifnamit ifit;

  for(;ifit();ifit.next())
  {
      const char *ifname=ifit();
      if(strstr(ifname,"isdn")!=NULL || strstr(ifname,"ppp")!=NULL)
      {
	  corbalog << ifname   << endl;
	  PortableServer::ObjectId_var oid=PortableServer::string_to_ObjectId(ifname);
	  CORBA::Object_var obj= interfpoa->create_reference_with_id(oid,"IDL:DialupManager/DialDevice:1.0");

	  try
	  {
	      CosNaming::Name name;
	      name.length(1);
	      name[0].id = CORBA::string_dup (ifname);
	      name[0].kind = CORBA::string_dup ("");
	      ddmnc->rebind(name,obj);
	  }
	  catch(CosNaming::NamingContext::NotFound_catch &nex)
	  {
	      corbalog.setprio(Syslog::Err);
	      corbalog << nex << ": da " << nex.why << endl;
	      corbalog.setprio(Syslog::Debug);
	  }
	  catch (CosNaming::NamingContext::CannotProceed_catch &exc)
	  {
	      corbalog.log(Syslog::Err,"CannotProceed exception");
	  }
	  catch (CosNaming::NamingContext::InvalidName_catch &exc)
	  {
	      corbalog.log(Syslog::Err,"InvalidName exception");
	  }
      } /*if */
  } /* for */
}

/*------------------- DialDeviceManager-----------------------------*/
class DialDeviceManager : public virtual POA_PortableServer::ServantActivator
{
public:  
  PortableServer::Servant incarnate (const PortableServer::ObjectId &,
                                     PortableServer::POA_ptr);
  void etherealize (const PortableServer::ObjectId &,
                    PortableServer::POA_ptr,
                    PortableServer::Servant,
                    CORBA::Boolean, CORBA::Boolean);

};

PortableServer::Servant DialDeviceManager::incarnate (const PortableServer::ObjectId &oid,
                                     PortableServer::POA_ptr poa)
{
    CORBA::String_var name=PortableServer::ObjectId_to_string(oid);
    DialDevice_impl *ret;
    char descdatpath[128];
    char descr[1024];
    char evref[1024];
    CORBA::Object_var myev_obj = CORBA::Object::_nil();
    CosEventChannelAdmin::EventChannel_ptr event_channel;
    if(main_debuglevel >0)
    {
	corbalog.setprio(Syslog::Debug);
	corbalog << "incarnate DialDevice " << name << endl;
    }

    snprintf(descdatpath,128,"%s%s",descdatprefix,(char *)name);
    
    try
    {
	ifstream descdat(descdatpath,ios::in);

	if(!descdat)
	{
	    corbalog.log(Syslog::Err,"DialDeviceManager::incarnate: Datei %s nicht gefunden !",descdatpath);
	    descr[0]='\0';
	}
	else
	{
	    corbalog.setprio(Syslog::Debug);
	    descdat.getline(descr,1024);
	    if(!descdat.eof())
	    {
		descdat >> evref;
		if(strncmp(evref,"IOR:",4)==0
		   || strncmp(evref,"iioploc:",8)==0
		   || strncmp(evref,"iiopnam:",8)==0)
		{
		    if(main_debuglevel>1)
		    {
			corbalog << "Referenz meines Eventchannels: " 
				 << evref << endl;
		    }
		    myev_obj = orb->string_to_object(evref);
		    if(myev_obj->_non_existent())
		    {
			if(main_debuglevel >1)
			{
			    corbalog << "Gibt es nicht mehr !!!!!" << endl;
			}
			myev_obj = CORBA::Object::_nil();
		    }
		    else
		    {
			event_channel = CosEventChannelAdmin::EventChannel::_narrow(myev_obj);
			if(CORBA::is_nil(event_channel))
			{
			    if(main_debuglevel >1)
			    {
				corbalog << "Kein Eventchannel " << endl;
			    }
			    myev_obj = CORBA::Object::_nil();
			}
		    }
		}
	    }
	}
    }
    catch(...)
    {
	myev_obj = CORBA::Object::_nil();
    }
    ret = new DialDevice_impl(name,descr,&isdnsystem);

    if(CORBA::is_nil(myev_obj))
    {
	corbalog << "Mu mir neuen Eventchannel erzeugen ... " << endl;
	assert(! CORBA::is_nil (eventchannelfactory));
	try
	{
	    event_channel = eventchannelfactory->create_eventchannel();
	}
	catch(...)
	{
	    corbalog.log(Syslog::Crit,"Kann eventchannel nicht erzeugen !!!");
	}
	
    }
    // Generiere Pushsupplier
    if(main_debuglevel > 0)
	corbalog << "Generiere Pushsupplier " << endl;
    POA_CosEventComm::PushSupplier_tie<DialDevice_impl> *pushsupsvt = 
	new POA_CosEventComm::PushSupplier_tie<DialDevice_impl>(ret,rootpoa.in(),FALSE);
    PortableServer::ObjectId_var pushsupoid = rootpoa->activate_object(pushsupsvt);
    
    ret->initpushsupplier(pushsupsvt->_this(),event_channel);
	
    ddsvmap[ret]=ret;
    return ret;
}

void DialDeviceManager::etherealize (const PortableServer::ObjectId &oid,
                    PortableServer::POA_ptr poa,
                    PortableServer::Servant serv,
                    CORBA::Boolean cleanup_in_progress, 
		    CORBA::Boolean remaining_activations)
{
    char descdatpath[128];
    
    corbalog.setprio(Syslog::Debug);
    CORBA::String_var nid=PortableServer::ObjectId_to_string(oid);
    snprintf(descdatpath,128,"%s%s",descdatprefix,(char *)nid);
    if(main_debuglevel > 0)
	corbalog << "etherealize " << nid << " in " << descdatpath << endl;

    if (!remaining_activations) 
    {
	DialDeviceServantMap::iterator it = ddsvmap.find(serv);
	assert(it!=ddsvmap.end());
	if(main_debuglevel>1) 	corbalog << "Speichere ... " << endl;
	(*it).second->store(descdatpath);
	if(main_debuglevel > 1)
	{
	    corbalog << " Verabschiede mich von Dialdevice "
		     << (*it).second->name() << endl;
	}
	ddsvmap.erase(it);
	delete serv;
    }
    else
    {
	corbalog << "Noch Aufrufe ... " << endl;
    }
}


void generalsauber()
{
    DialDeviceServantMap::iterator it;
    if(main_debuglevel > 1)
    {
	corbalog.log(Syslog::Debug,"generalsauber:Hnge alle Pushsupplier ab ...");
    }
    for(it=ddsvmap.begin();it!=ddsvmap.end();++it)
    {
	(*it).second->cleanup();
    }
}

void sighandler (int sig)
{
  
  if (!CORBA::is_nil (orb))
  {
    generalsauber();  
    orb->shutdown (TRUE);
  }
}

const char *poastate(PortableServer::POAManager_ptr mgr)
{
    PortableServer::POAManager::State mgrstate;
    mgrstate = mgr->get_state();
    switch(mgrstate)
    {
    case PortableServer::POAManager::HOLDING:    return "HOLDING"; 
    case PortableServer::POAManager::ACTIVE:     return "ACTIVE"; 
    case PortableServer::POAManager::DISCARDING: return "DISCARDING"; 
    case PortableServer::POAManager::INACTIVE:   return "INACTIVE"; 
    }
}

void sigusrhandler(int sig)
{
    corbalog.setprio(Syslog::Debug);
    corbalog << "SIGUSR1" << endl;

    PortableServer::POAManager_var rootmgr   = rootpoa->the_POAManager();
    PortableServer::POAManager_var interfmgr = interfpoa->the_POAManager();
    
    corbalog << "Root-POA ist "      << poastate(rootmgr)   << endl;
    corbalog << "Interface-POA ist " << poastate(interfmgr) << endl;
}

void installsigusrhandler()
{
    struct sigaction action;
    action.sa_handler = sigusrhandler;
    sigemptyset(&(action.sa_mask));
    sigaddset(&(action.sa_mask),SIGTERM);
    sigaddset(&(action.sa_mask),SIGINT);
    sigaddset(&(action.sa_mask),SIGUSR1);
    
    action.sa_flags = SA_RESTART;
    if(sigaction(SIGUSR1,&action,NULL)<0) perror("sigaction(SIGUSR1)");
    
}

void usage(char *progname)
{
    cerr << "Aufruf: " << progname
	 << "[mico-args ...] "
	 << endl;
    exit(1);
}

int main(int argc,char **argv)
{
  orb = CORBA::ORB::_nil ();
  rootpoa = PortableServer::POA::_nil();
  interfpoa = PortableServer::POA::_nil();
  signal (SIGINT, sighandler);
  signal (SIGTERM, sighandler);
  installsigusrhandler();
  char *prog = rindex(argv[0],'/');
  prog = (prog ? prog+1 : argv[0]);

  // POA-Name faelschen
  const char *implname = NULL;
  char **orbargv = new char*[argc+2+1];
  int orbargc=0;

  for(int i=0;i<argc;i++)
  {
    if(strcmp(argv[i],"-POAImplName")==0 && i+1<argc)
    {
	implname = argv[i+1];
    }
    // das muss ein Orb-argument sein ...
    orbargv[orbargc++]=argv[i];
  }
  if(implname==NULL)
  {
      orbargv[orbargc++]="-POAImplName";
      orbargv[orbargc++]="DialupManager";
  }
  orbargv[orbargc]=NULL;

  cerr << "ORB und POA initialisieren ... " << endl;
  // Orb und POA initialisieren
  orb = CORBA::ORB_init(orbargc,orbargv,"mico-local-orb");
  CORBA::Object_var poaobj = orb->resolve_initial_references("RootPOA");
  rootpoa = PortableServer::POA::_narrow(poaobj);
  PortableServer::POAManager_var mgr = rootpoa->the_POAManager();
  

  try
  {
      CORBA::PolicyList pl;
      pl.length(5);
      int p=0;
      pl[p++] = rootpoa->create_request_processing_policy(PortableServer::USE_SERVANT_MANAGER);
      // pl[p++] = rootpoa->create_id_uniqueness_policy(PortableServer::MULTIPLE_ID);
      pl[p++] = rootpoa->create_id_assignment_policy(PortableServer::USER_ID);
      pl[p++] = rootpoa->create_implicit_activation_policy(PortableServer::IMPLICIT_ACTIVATION);
      pl[p++] = rootpoa->create_lifespan_policy(PortableServer::PERSISTENT);
      pl.length(p);
      
      interfpoa = rootpoa->create_POA("Interfaces",mgr,pl);
  }
  catch(PortableServer::POA::InvalidPolicy_catch &ex)
  {
      cerr << "InvalidPolicy: ";
      ex->_print(cerr);
      cerr << " Index " << ex->index << endl;
      exit(1);
  }
  catch (CORBA::SystemException_catch &ex)
  {
      cout << ex->minor() << endl;
  }

  DialDeviceManager *ddmanager = new DialDeviceManager;
  PortableServer::ServantManager_var ddmanagerref = ddmanager->_this();
  interfpoa->set_servant_manager(ddmanagerref);


  mgr->activate();
  
  /*----- Initialisiere Eventfactory -----------------------*/
  CosNaming::Name efname;
  CORBA::Object_var evfobj;
  CORBA::Object_var nsobj = orb->resolve_initial_references("NameService");
  if(CORBA::is_nil(nsobj))
  {
    cerr << "Hallo: Initial Service NameService not resolved " << endl;
    exit(1);
  }
  CosNaming::NamingContext_var nc =CosNaming::NamingContext::_narrow (nsobj);


  try
  {
      evfobj =nc->resolve(StringToName(efname,"EventChannelFactory"));
  }
  catch(...)
  {
      cerr << "Kann EventChannelFactory nicht finden !! " << endl;
      exit(1);
  }
      
  eventchannelfactory = SimpleEventChannelAdmin::EventChannelFactory::_narrow(evfobj);
  assert(! CORBA::is_nil (eventchannelfactory));

  cerr << "----- ffne Corba-log --------" << endl;
  CosEventChannelAdmin::EventChannel_var syslogev;
 
  try
  {
      CosNaming::Name syslogevname;
      CORBA::Object_var syslogevobj=nc->resolve(StringToName(syslogevname,"syslog"));
      syslogev = CosEventChannelAdmin::EventChannel::_narrow(syslogevobj);
      if(CORBA::is_nil (syslogev))
      {
	  // throw
	  cerr << "syslog hat falschen typ " << endl;
	  exit(1);
      }
  }
  catch(...)
  {
      cerr << "Kann syslog nicht finden !! " << endl;
      exit(1);
  }
      
  corbalog.init(orb,syslogev.in(),"corbammims",prog);
#ifdef LIBSTD
  Control_impl *controlservant = new DmControl_impl(prog);
  controlservant->init_debugtab();
#else
  DmControl_impl *controlservant = new DmControl_impl(prog,dvmt,sizeof(dvmt)/sizeof(struct Debugvar));
#endif

  rootpoa->activate_object(controlservant);
  {
      char ctlpath[128];
      snprintf(ctlpath,128,"%s%s.ctlref",RUNPATH,prog);
      ofstream aus(ctlpath);
      assert(aus);
      aus << orb->object_to_string(controlservant->_this());
  }
  IsdnCTIEventCallback *ctipushsup = new IsdnCTIEventCallback(&isdnsystem);
  rootpoa->activate_object(ctipushsup);
  ctipushsup->opencti(eventchannelfactory.in());
  cticonsumeradm=ctipushsup->getconsumeradmin();

  /* Liste von Objekten mit exportname() ? */
  controlservant->exportnames(nc);
  /*------- Isdnereignislauscher einbinden------------*/
  cerr << "Wir hngen unsere Isdnereignisbehandlung ein ... " << endl;
  IsdnEventCallback *iecb = new IsdnEventCallback(&isdnsystem,&ddsvmap);
  orb->dispatcher()->rd_event(iecb,isdnsystem.infofd());
#ifdef MIT_ISDN_LOG
  IsdnCtrlEventCallback *icecb = new IsdnCtrlEventCallback(&isdnsystem);
  orb->dispatcher()->rd_event(icecb,isdnsystem.ctrlfd());
#endif
  orb->dispatcher()->rd_event(ctipushsup,ctipushsup->ctifd());
  /*------------- Starte Orb -------------------------*/
  orb->run();
}











