#include"dialupmanager.h"
#include<mico/CosEventChannelAdmin.h>
#include"pushconsumer_impl.h"
#include<mico/qtmico.h>
#include<qlayout.h>
#include<qpushbutton.h>
#include<qlabel.h>
#include<qcombobox.h>
#include<strstream.h>
#include<qtextstream.h>
#include<qtimer.h>

class QtDialMonitor : public QWidget,public Erfrischbar
{
    Q_OBJECT
 public:
    QtDialMonitor(DialupManager::DialDevice_ptr _dd,QWidget *parent = 0,const char *name=0);
    void verbinde();
    void erfrische(const CORBA::Any& data);
    void disconnect(); // Unser Pushsupplierer wurde beendet
    
    void dispstatus(DialupManager::DialDevice::Status st);
    void dispdialmode(DialupManager::DialDevice::DialMode dm);
 signals:
 public slots:
    void setdialmode(int dm);
    void aendaktiv();
    void quit();
    void fetchip();
 protected:    
    QLabel *wname;
    QLabel *wdesc;
    QComboBox *dialupcombo;
    QLabel *status;
    QPushButton *dial;
    QLabel *msg;
    
    DialupManager::DialDevice_var dd;
    CosEventComm::PushConsumer_var mein_pc;
    CosEventChannelAdmin::ProxyPushSupplier_var der_ps;
    int aktivzustand;

    QTimer *fiptimer;
};

const char *dialmode[] = { "off","manual","automatic","permanent",0 };
const char *statusstr[] = { "offline","incoming","outgoing",0 };
const char *dialbuttonstr[] = { "whlen","auflegen",0};

QtDialMonitor::QtDialMonitor(DialupManager::DialDevice_ptr _dd,QWidget *parent = 0,const char *name=0)
{
    dd=_dd;
    QBoxLayout *topLayout = new QVBoxLayout( this,2);
    wname=new QLabel(this,"name");
    wname->setFont(QFont("Helvetica",16,QFont::Bold));
    wname->setAlignment(AlignHCenter);
    topLayout->addWidget(wname);
   
    wdesc=new QLabel(this,"desc");
    wdesc->setAlignment(AlignHCenter);
    topLayout->addWidget(wdesc);

    dialupcombo= new QComboBox(FALSE,this,"dialmode");
    dialupcombo->insertStrList(dialmode);
    topLayout->addWidget(dialupcombo);
    connect(dialupcombo,SIGNAL(activated(int)),this,SLOT(setdialmode(int)));

    status = new QLabel(this,"status");
    status->setAlignment(AlignHCenter);
    topLayout->addWidget(status);

    dial = new QPushButton("Aktiviere",this,"dial");
    topLayout->addWidget(dial);
    connect(dial,SIGNAL(clicked()),this,SLOT(aendaktiv()));

    QPushButton *quit= new QPushButton("Quit",this,"quit");
    topLayout->addWidget(quit);
    
    msg = new QLabel(this,"");
    msg->setAlignment(AlignHCenter);
    topLayout->addWidget(msg);

    connect(quit,SIGNAL(clicked()),this,SLOT(quit()));

    topLayout->activate();

    /* IP kann erst spter geholt werden */
    fiptimer= new QTimer(this);
    connect(fiptimer,SIGNAL(timeout()),this,SLOT(fetchip()));
}

void QtDialMonitor::verbinde()
{
    QString sname;
    QString sdesc;
    PushConsumer_impl *pci;
    ostrstream errmsgstr;
    QString errmsg;

    try
    {
	sname.sprintf(dd->name());
	wname->setText(sname);
	sdesc.sprintf(dd->desc());
	wdesc->setText(sdesc);
    }
    catch(CORBA::SystemException_catch &ex)
    {
	errmsgstr << "Beim Ermitteln der Informationen ";
	ex._print(errmsgstr);
	errmsg.sprintf(errmsgstr.str());
	msg->setText(errmsg);
	return;
    }

    CosEventChannelAdmin::EventChannel_var eventchannel;
    CosEventChannelAdmin::ConsumerAdmin_var consumeradmin;
    try
    {
	pci=new PushConsumer_impl(this);
	mein_pc = pci->_this();	

	eventchannel=dd->geteventchannel();
	assert(!CORBA::is_nil(eventchannel));
	consumeradmin=eventchannel->for_consumers();
	assert(!CORBA::is_nil(consumeradmin));
	der_ps = consumeradmin->obtain_push_supplier();
	assert(!CORBA::is_nil(der_ps));
	der_ps->connect_push_consumer(mein_pc.in());
    }
    catch(CORBA::SystemException_catch &ex)
    {
	errmsgstr << "Verbinde Eventchannel ";
	ex._print(errmsgstr);
	errmsg.sprintf(errmsgstr.str());
	msg->setText(errmsg);
	return;
    }
}

void QtDialMonitor::erfrische(const CORBA::Any& data)
{
    ostrstream errmsgstr;
    QString errmsg;

    DialupManager::dmstate dms;

    try
    {
	if(data >>= dms)
	{
	    // cerr << "Kann Ereignisdaten verwenden " << endl;
	}
	else
	{
	    // cerr << "Mu Zustand nachfragen ... " << endl;
	    dms.dm=dd->getdialmode();
	    dms.status=dd->getstatus();
	}
	
	dispstatus(dms.status);
	dispdialmode(dms.dm);
	CORBA::String_var ipstr = dd->get_our_ip();
	QTextOStream(&errmsg) << "IP: " << ipstr;
	msg->setText(errmsg);
	// cerr << "Bestelle fetchip ... " << endl;
	fiptimer->start(5000,TRUE);
    }
    catch(DialupManager::Syserr& ex)
    {
	errmsgstr << "Syserr:";ex._print(errmsgstr); 
	errmsg.sprintf(errmsgstr.str());
	msg->setText(errmsg);
    }
    catch(CORBA::SystemException_catch &ex)
    {
	errmsgstr << "erfrische: ";
	ex._print(errmsgstr);
	errmsg.sprintf(errmsgstr.str());
	msg->setText(errmsg);
	return;
    }
}

void QtDialMonitor::fetchip()
{
    ostrstream errmsgstr;
    QString errmsg;

    // cerr << "fetchip" << endl;
    try
    {
	CORBA::String_var ipstr = dd->get_our_ip();
	QTextOStream(&errmsg) << "IP: " << ipstr;
	msg->setText(errmsg);
    }
    catch(DialupManager::Syserr& ex)
    {
	errmsgstr << "Syserr:";ex._print(errmsgstr); 
	errmsg.sprintf(errmsgstr.str());
	msg->setText(errmsg);
    }
    catch(CORBA::SystemException_catch &ex)
    {
	errmsgstr << "erfrische: ";
	ex._print(errmsgstr);
	errmsg.sprintf(errmsgstr.str());
	msg->setText(errmsg);
	return;
    }
}

void QtDialMonitor::disconnect()
{
    QString errmsg;
    errmsg.sprintf("Pushsupplier wurde abgehngt");
    msg->setText(errmsg);
}

void QtDialMonitor::dispstatus(DialupManager::DialDevice::Status st)
{
    int s=(int)st;
    aktivzustand = (s>0);
    QString sstr = statusstr[st];
    QString dbl  = dialbuttonstr[aktivzustand];
    status->setText(sstr);
    dial->setText(dbl);
    
}

void QtDialMonitor::dispdialmode(DialupManager::DialDevice::DialMode dm)
{
    dialupcombo->setCurrentItem((int)dm);
    
}

void QtDialMonitor::setdialmode(int d)
{
    ostrstream errmsgstr;
    QString errmsg;

    // cout << "Set Dialmode " << d << endl;
    try
    {
	dd->setdialmode((DialupManager::DialDevice::DialMode)d);
    }
    catch(DialupManager::Syserr& ex)
    {
	errmsgstr << "Syserr:";ex._print(errmsgstr); 
	errmsg.sprintf(errmsgstr.str());
	msg->setText(errmsg);
    }
    catch(CORBA::SystemException_catch &ex)
    {
	errmsgstr << "Bei erfrische: ";
	ex._print(errmsgstr);
	errmsg.sprintf(errmsgstr.str());
	msg->setText(errmsg);
	return;
    }
}

void QtDialMonitor::aendaktiv()
{
    ostrstream errmsgstr;
    QString errmsg;

    try
    {
	if(aktivzustand)
	{
	    // cerr << "deaktiviere " << endl;
	    dd->deactivate();
	}
	else
	{
	    // cerr << "aktiviere " << endl;
	    dd->activate();
	}
    }
    catch(DialupManager::Syserr& ex)
    {
	errmsgstr << "Syserr:";
	ex._print(errmsgstr); 
	errmsg.sprintf(errmsgstr.str());
	msg->setText(errmsg);
    }
    catch(CORBA::SystemException_catch &ex)
    {
	errmsgstr << "aendaktiv: ";
	ex._print(errmsgstr);
	errmsg.sprintf(errmsgstr.str());
	msg->setText(errmsg);
	return;
    }
}

void QtDialMonitor::quit()
{
    // cerr << "Und Tsch ... " << endl;
    der_ps->disconnect_push_supplier();
    qApp->quit();
}

#include"qtdialmonitor.moc"
#include<iostream.h>

int main(int argc,char **argv)
{
    CApp a( argc, argv );
    if(a.argc()<2)
    {
	cerr << "Aufruf: qtdialupmonitor <objid oder url>" << endl;
	exit(1);
    }
    const char *url= a.argv()[1];
    CORBA::Object_var ddobj = a.orb->string_to_object(url);
    if(CORBA::is_nil(ddobj))
    {
	cerr << url << " nicht referenzierbar " << endl;
	exit(1);
    }
    DialupManager::DialDevice_ptr dd=DialupManager::DialDevice::_narrow(ddobj);
    if(CORBA::is_nil(dd))
    {
	cerr << url << " ist ein " << ddobj->_ior()->objid() << " und kein IDL:DialupManager/DialDevice:1.0" << endl;
	exit(1);
    }

    CORBA::Any leer;
    leer <<= (CORBA::Long)0;
    QtDialMonitor *qdm = new QtDialMonitor(dd,0,"Dialmonitor");
    a.setMainWidget(qdm);
    qdm->show();
    qdm->verbinde();
    qdm->erfrische(leer);
    
    
    a.impl_is_ready(CORBA::ImplementationDef::_nil());
    a.run();
}





