#include"dialupmanager.h"
#include<mico/qtmico.h>
#include<qlayout.h>
#include<qpushbutton.h>
#include<qlabel.h>
#include<qdialog.h>
#include<qmultilineedit.h>
#include<strstream.h>

class ShowmailDialog: public QDialog
{
    Q_OBJECT
public:
    ShowmailDialog(QWidget *parent,const char *name=0);
    void append(const char *zeile);
    void clear();

protected:
    QMultiLineEdit *mle;
};

class QtQueueMonitor: public QWidget
{
    Q_OBJECT
public:
    QtQueueMonitor(DialupManager::Queuemanager_ptr qm,QWidget *parent = 0,const char *name=0);
    QLabel *msg;
    void verbinde();


signals:
public slots:
    void fetchandsend();
    void erfrische();
    void showqueue();
    void quit();
protected:
    QLabel *wname;
    QLabel *wdesc;
    QLabel *wlen;
    
    ShowmailDialog *smdialog;
    DialupManager::Queuemanager_var qm;
};

/*-------------- ShowmailDialog ----------------------*/
ShowmailDialog::ShowmailDialog(QWidget *parent,const char *name):QDialog(parent,name,FALSE)
{
    QString stitel = "Warteschlange";
    QBoxLayout *topLayout = new QVBoxLayout(this,3);
    QLabel *titel = new QLabel(stitel,this,"titel");
    titel->setAlignment(AlignHCenter);
    topLayout->addWidget(titel);
    mle = new QMultiLineEdit(this,"anzeige");
    mle->setMinimumSize(560,160);
    mle->setReadOnly(TRUE);
    topLayout->addWidget(mle);

    QPushButton *cancel = new QPushButton("Cancel",this);
    topLayout->addWidget(cancel);
    
    connect(cancel,SIGNAL(clicked()),SLOT(reject()));
}

void ShowmailDialog::append(const char *zeile)
{
    mle->append(zeile);
}

void ShowmailDialog::clear()
{
    mle->clear();
}
/*-------------- QtQueueMonitor ----------------------*/
QtQueueMonitor::QtQueueMonitor(DialupManager::Queuemanager_ptr pqm,QWidget *parent,const char *name)
{
    qm=pqm;
    QBoxLayout *topLayout = new QVBoxLayout(this,2);
    wname= new QLabel(this,"name");
    wname->setFont(QFont("Helvetica",16,QFont::Bold));
    wname->setAlignment(AlignHCenter);
    topLayout->addWidget(wname);
   
    wdesc=new QLabel(this,"desc");
    wdesc->setAlignment(AlignHCenter);
    topLayout->addWidget(wdesc);
    
    QBoxLayout *lenmitlab = new QHBoxLayout(topLayout,5);
    topLayout->addLayout(lenmitlab);

    QLabel *wlenl = new QLabel("Lnge",this,"queuelengthlabel");
    lenmitlab->addWidget(wlenl);

    wlen = new QLabel(this,"queuelength");
    lenmitlab->addWidget(wlen);

    QBoxLayout *knopfleiste = new QHBoxLayout(topLayout,5);
    topLayout->addLayout(knopfleiste);

    QPushButton *refresh = new QPushButton("Erfrische",this,"erfrisch");
    knopfleiste->addWidget(refresh);
    connect(refresh,SIGNAL(clicked()),this,SLOT(erfrische()));

    QPushButton *hauweg = new QPushButton("Hole und hau weg",this,"hauweg");
    knopfleiste->addWidget(hauweg);
    connect(hauweg,SIGNAL(clicked()),this,SLOT(fetchandsend()));

    QPushButton *zeigeschl = new QPushButton("Zeige Warteschlange",this,"zeigewarteschlange");
    knopfleiste->addWidget(zeigeschl);
    connect(zeigeschl,SIGNAL(clicked()),this,SLOT(showqueue()));

    QPushButton *quit = new QPushButton("Ende",this,"ende");
    knopfleiste->addWidget(quit);
    connect(quit,SIGNAL(clicked()),this,SLOT(quit()));
 
    lenmitlab->activate();
    knopfleiste->activate();
    msg = new QLabel(this,"");
    msg->setAlignment(AlignHCenter);
    topLayout->addWidget(msg);

    topLayout->activate();

    smdialog = new ShowmailDialog(this,"showmaildialog");
}

void QtQueueMonitor::verbinde()
{
    QString sname;
    QString sdesc;
    QString errmsg;
    ostrstream errmsgstr;

    try
    {
	sname.sprintf(qm->name());
	wname->setText(sname);
	sdesc.sprintf(qm->desc());
	wdesc->setText(sdesc);
    }
    catch(DialupManager::Syserr_catch &ex)
    {
	ex._print(errmsgstr);errmsg.sprintf(errmsgstr.str());msg->setText(errmsg);
    }
    catch(CORBA::SystemException_catch &ex)
    {
	errmsgstr << "Bei erfrische: ";
	ex._print(errmsgstr);
	errmsg.sprintf(errmsgstr.str());msg->setText(errmsg);
	return;
    }
}

void QtQueueMonitor::fetchandsend()
{
    QString errmsg;
    ostrstream errmsgstr;

    // cerr << "fetchandsend" << endl;
    try
    {
	qm->fetchandsend();
    }
    catch(DialupManager::Syserr_catch &ex)
    {
	ex._print(errmsgstr);
	errmsg.sprintf(errmsgstr.str());msg->setText(errmsg);
    }
    catch(CORBA::SystemException_catch &ex)
    {
	errmsgstr << "fetch: ";
	ex._print(errmsgstr);
	errmsg.sprintf(errmsgstr.str());msg->setText(errmsg);
	return;
    }
}

void QtQueueMonitor::erfrische()
{
    QString errmsg;
    ostrstream errmsgstr;

    try
    {
	wlen->setNum((int)qm->howmanyentries());
    }
    catch(DialupManager::Syserr_catch &ex)
    {
	errmsgstr << "erfrische";
	ex._print(errmsgstr);
	errmsg.sprintf(errmsgstr.str());msg->setText(errmsg);
    }
    catch(CORBA::SystemException_catch &ex)
    {
	errmsgstr << "erfrische: ";
	ex._print(errmsgstr);
	errmsg.sprintf(errmsgstr.str());msg->setText(errmsg);
	return;
    }
}

void QtQueueMonitor::showqueue()
{
    QString errmsg;
    ostrstream errmsgstr;

    DialupManager::EntrydescList_var l;
    DialupManager::QueueentryIterator_var quei;
    //cerr << "showqueue" << endl;
    //smdialog->exec();
    smdialog->show();
    smdialog->clear();
    try
    {
	qm->listentries(2,l,quei);
	for(int i=0;i<l->length();i++)
	{
	    smdialog->append((*l)[i]);
	}
	if(!CORBA::is_nil(quei))
	{
	    while(quei->next_n(10,l))
	    {
		for(int i=0;i<l->length();i++)
		{
		    smdialog->append((*l)[i]);
		}			
	    }
	    quei->destroy();
	}

    }
    catch(DialupManager::Syserr_catch &ex)
    {
	errmsgstr << "showqueue";
	ex._print(errmsgstr);
	errmsg.sprintf(errmsgstr.str());msg->setText(errmsg);
    }
    catch(CORBA::SystemException_catch &ex)
    {
	errmsgstr << "Bei erfrische: ";
	ex._print(errmsgstr);
	errmsg.sprintf(errmsgstr.str());msg->setText(errmsg);
	return;
    }
}

void QtQueueMonitor::quit()
{
    // cerr << "Und Tsch" << endl;
    qApp->quit();
}
#include"qtqueuemonitor.moc"

int main(int argc,char **argv)
{
    CApp a( argc, argv );
    if(a.argc()<2)
    {
	cerr << "Aufruf: qtdialupmonitor <objid oder url>" << endl;
	exit(1);
    }
    const char *url= a.argv()[1];
    CORBA::Object_var qmobj = a.orb->string_to_object(url);
    if(CORBA::is_nil(qmobj))
    {
	cerr << url << " nicht referenzierbar " << endl;
	exit(1);
    }
    DialupManager::Queuemanager_ptr qm=DialupManager::Queuemanager::_narrow(qmobj);
    if(CORBA::is_nil(qm))
    {
	cerr << url << " ist ein " << qmobj->_ior()->objid() << " und kein IDL:DialupManager/Queuemanager:1.0" << endl;
	exit(1);
    }
	
    QtQueueMonitor *qqm = new QtQueueMonitor(qm,0,"qtQueuemonitor");
    a.setMainWidget(qqm);
    qqm->show();
    qqm->verbinde();
    qqm->erfrische();
    a.impl_is_ready(CORBA::ImplementationDef::_nil());
    a.run();
}










