#include<iostream.h>
#include<fstream.h>
#include<signal.h>
#include<string.h>
#include<unistd.h>
#include<mico/CosNaming.h>
#include"queuemanager_impl.h"
#include"queuepoa.h"
#include"corbalogger.h"
#include"control_impl.h"
#include"dirit.h"
#include"config.h"

int main_debuglevel=0;
CorbaLog corbalog;

char *descdatprefix = DATAPATH;
CORBA::ORB_ptr orb;
PortableServer::POA_var rootpoa;
PortableServer::POA_var queuempoa;

SimpleEventChannelAdmin::EventChannelFactory_var eventchannelfactory;

/*------------------- Control-Interface ----------------------------*/
class DmControl_impl: public Control_impl
{
public:
#ifdef LIBSTD
    DmControl_impl::DmControl_impl(const char *pname):Control_impl(pname) {}
    void init_debugtab();
#else
    DmControl_impl::DmControl_impl(const char *pname,Debugvarmap _dvm,int dvmsz):Control_impl(pname,_dvm,dvmsz) {}
#endif
  void exportnames( CosNaming::NamingContext_ptr nc) 
      throw(::CORBA::SystemException);
};

extern int main_debuglevel;
extern int queuemanager_debuglevel;
extern int queuepoa_debuglevel;

#ifdef LIBSTD  
void DmControl_impl::init_debugtab()
{
   dvm["main"] = &main_debuglevel;
   dvm["queuemanager"] = &queuemanager_debuglevel;
   dvm["queuepoa"] = &queuepoa_debuglevel;
}
#else
Debugvar dvmt[]=
{
   {"main",&main_debuglevel},
   {"queuemanager",&queuemanager_debuglevel},
   {"queuepoa",&queuepoa_debuglevel},
   {NULL,NULL}
};
#endif

void
DmControl_impl::exportnames(CosNaming::NamingContext_ptr nc )
  throw(::CORBA::SystemException)

{
    /*---- Verffentliche unsere Queues im Nameserver ----*/  
  CosNaming::Name contextname;
  contextname.length (1);
  contextname[0].id = CORBA::string_dup ("DialupManager");
  contextname[0].kind = CORBA::string_dup ("");
  CORBA::Object_var ddmnc_obj;
  CosNaming::NamingContext_var ddmnc;
    
  corbalog << "Verffentliche unsere Queues im Nameserver ... " << endl;
    try
  {
      ddmnc_obj = nc->bind_new_context(contextname);
      if(CORBA::is_nil(ddmnc_obj))
      {
	  corbalog.log(Syslog::Err,"Dialupmanager-Kontext ist NIL");
	  return;
      }
      ddmnc=CosNaming::NamingContext::_narrow(ddmnc_obj);
  }
  catch(CosNaming::NamingContext::AlreadyBound_catch &nex)
  {
      corbalog << "Hoppla: Schon gebunden !" << endl;
      ddmnc_obj = nc->resolve(contextname);
      ddmnc=CosNaming::NamingContext::_narrow(ddmnc_obj);
  }
  catch(CosNaming::NamingContext::NotFound_catch &nex)
  {
      corbalog.setprio(Syslog::Err);
      corbalog << nex << ": da " << nex.why << endl;
      corbalog.setprio(Syslog::Debug);
      return;
  }
  catch (CosNaming::NamingContext::CannotProceed_catch &exc)
  {
      corbalog.log(Syslog::Err,"CannotProceed exception");
      return;
  }
  catch (CosNaming::NamingContext::InvalidName_catch &exc)
  {
      corbalog.log(Syslog::Err,"InvalidName exception");
      return;
  }

    /*----------------- Trage die Queues ein -----------------*/
  corbalog << "Queues:" << endl;
  Dirit dirit(descdatprefix,".queue");

  for(;dirit();dirit.next())
  {
    const char *qname=dirit();
    char *endung=strstr(qname,".queue");
    *endung='\0';
    corbalog << qname   << endl;
    PortableServer::ObjectId_var oid=PortableServer::string_to_ObjectId(qname);
    CORBA::Object_var obj= queuempoa->create_reference_with_id(oid,"IDL:DialupManager/Queuemanager:1.0");
    // cerr << orb->object_to_string(obj) << endl;
    // obj->_ior()->print(cout);
    try
    {
	CosNaming::Name name;
	name.length(1);
	name[0].id = CORBA::string_dup (qname);
	name[0].kind = CORBA::string_dup(""); // ("queue"); nicht sichtbar
	ddmnc->rebind(name,obj);
    }
    catch(CosNaming::NamingContext::NotFound_catch &nex)
    {
	corbalog.setprio(Syslog::Err);
	corbalog << nex << ": da " << nex.why << endl;
	corbalog.setprio(Syslog::Debug);
    }
    catch (CosNaming::NamingContext::CannotProceed_catch &exc)
    {
	corbalog.log(Syslog::Err,"CannotProceed exception");
    }
    catch (CosNaming::NamingContext::InvalidName_catch &exc)
    {
	corbalog.log(Syslog::Err,"InvalidName exception");
    }
  } /* for */
}

void sighandler (int sig)
{
  
  if (!CORBA::is_nil (orb))
  {
      // generalsauber();  
      orb->shutdown (TRUE);
  }
}

void usage(char *progname)
{
    cerr << "Aufruf: " << progname
	 << "[mico-args ...] "
	 << endl;
    exit(1);
}

int main(int argc,char **argv)
{
  orb = CORBA::ORB::_nil ();
  rootpoa = PortableServer::POA::_nil();
  signal (SIGINT, sighandler);
  signal (SIGTERM, sighandler);
  char *prog = rindex(argv[0],'/');
  prog = (prog ? prog+1 : argv[0]);

  // POA-Name faelschen
  const char *implname = NULL;
  char **orbargv = new char*[argc+2+1];
  int orbargc=0;

  for(int i=0;i<argc;i++)
  {
    if(strcmp(argv[i],"-POAImplName")==0 && i+1<argc)
    {
	implname = argv[i+1];
    }
    // das muss ein Orb-argument sein ...
    orbargv[orbargc++]=argv[i];
  }
  if(implname==NULL)
  {
      orbargv[orbargc++]="-POAImplName";
      orbargv[orbargc++]="DialQueueManager";
  }
  orbargv[orbargc]=NULL;

  cerr << "ORB und POA initialisieren ... " << endl;
  // Orb und POA initialisieren
  orb = CORBA::ORB_init(orbargc,orbargv,"mico-local-orb");
  CORBA::Object_var poaobj = orb->resolve_initial_references("RootPOA");
  rootpoa = PortableServer::POA::_narrow(poaobj);
  PortableServer::POAManager_var mgr = rootpoa->the_POAManager();
  

  try
  {
      CORBA::PolicyList pl;
      pl.length(5);
      int p=0;
      pl[p++] = rootpoa->create_request_processing_policy(PortableServer::USE_SERVANT_MANAGER);
      // pl[p++] = rootpoa->create_id_uniqueness_policy(PortableServer::MULTIPLE_ID);
      pl[p++] = rootpoa->create_id_assignment_policy(PortableServer::USER_ID);
      pl[p++] = rootpoa->create_implicit_activation_policy(PortableServer::IMPLICIT_ACTIVATION);
      pl[p++] = rootpoa->create_lifespan_policy(PortableServer::PERSISTENT);
      pl.length(p);
      
      queuempoa = rootpoa->create_POA("Queues",mgr,pl);
  }
  catch(PortableServer::POA::InvalidPolicy_catch &ex)
  {
      cerr << "InvalidPolicy: ";
      ex->_print(cerr);
      cerr << " Index " << ex->index << endl;
      exit(1);
  }
  catch (CORBA::SystemException_catch &ex)
  {
      cout << ex->minor() << endl;
  }


  QueuePOAManager *qmmanager = new QueuePOAManager;
  PortableServer::ServantManager_var qmmanagerref = qmmanager->_this();
  queuempoa->set_servant_manager(qmmanagerref);

  mgr->activate();
  
  /*----- Nameserver ermitteln und Corbalog ffnen -----*/
  CosNaming::Name efname;
  CORBA::Object_var nsobj = orb->resolve_initial_references("NameService");
  if(CORBA::is_nil(nsobj))
  {
    cerr << "Hallo: Initial Service NameService not resolved " << endl;
    exit(1);
  }
  CosNaming::NamingContext_var nc =CosNaming::NamingContext::_narrow (nsobj);

   cerr << "----- ffne Corba-log --------" << endl;
  CosEventChannelAdmin::EventChannel_var syslogev;
 
  try
  {
      CosNaming::Name syslogevname;
      CORBA::Object_var syslogevobj=nc->resolve(StringToName(syslogevname,"syslog"));
      syslogev = CosEventChannelAdmin::EventChannel::_narrow(syslogevobj);
      if(CORBA::is_nil (syslogev))
      {
	  // throw
	  cerr << "syslog hat falschen typ " << endl;
	  exit(1);
      }
  }
  catch(...)
  {
      cerr << "Kann syslog nicht finden !! " << endl;
      exit(1);
  }
      
  corbalog.init(orb,syslogev.in(),"corbammims",prog);
#ifdef LIBSTD
  Control_impl *controlservant = new DmControl_impl(prog);
  controlservant->init_debugtab();
#else
  DmControl_impl *controlservant = new DmControl_impl(prog,dvmt,sizeof(dvmt)/sizeof(struct Debugvar));
#endif

  rootpoa->activate_object(controlservant);
  {
      char ctlpath[128];
      snprintf(ctlpath,128,"%s%s.ctlref",RUNPATH,prog);
      ofstream aus(ctlpath);
      assert(aus);
      aus << orb->object_to_string(controlservant->_this());
  }

  controlservant->exportnames(nc);

  /*------------- Starte Orb -------------------------*/
  orb->run();
}  



