#define __USE_GNU
#include<iostream.h>
#include<fstream.h>
#include<stdlib.h>
#include<string.h>
#include"queuemanager_impl.h"
#include"corbalogger.h"
#include<sys/errno.h>
extern CorbaLog corbalog;

int queuemanager_debuglevel=0;

// Implementation for interface Queuemanager
Queuemanager_impl::Queuemanager_impl(const char *pname):POA_DialupManager::Queuemanager()
{
    iname=pname; 
    ddlwurzel=NULL;
}

char*
Queuemanager_impl::name()
  throw(
    ::CORBA::SystemException)

{
  char* retval;
  retval = CORBA::string_dup(iname);

  return retval; 
}


char*
Queuemanager_impl::desc()
  throw(
    ::CORBA::SystemException)

{
  char* retval;
  retval = CORBA::string_dup(idesc);
  return retval; 
}

void
Queuemanager_impl::fetchandsend()
  throw(
    ::CORBA::SystemException,
    DialupManager::Syserr)

{
    int fd;
    char puffer[1024];
    int l;
    Process proc;
    corbalog.setprio(Syslog::Notice);
    corbalog << "-------" << iname << " fetchandsend() --------" << endl;
    opendevices();
    fd=proc.openreadpipe(uid,fetchcmd);
    if(fd<0)
    {
	snprintf(puffer,1024,"openreadpipe:%s",strerror(errno));
	mico_throw(DialupManager::Syserr(errno,puffer));
    }
    if(fd==0)
    {
	corbalog << "fetchandsend: openreadpipe lieferte 0 " << endl;
    }
    while((l=read(fd,puffer,1024))>0)
    {
	puffer[l]='\0';
	corbalog << puffer << endl;	
    }
    if(l<0) { corbalog.log(Syslog::Err,"fetchandsend: read: %m"); }
    close(fd);
    l=proc.wait();
    closedevices();
    if(l!=0)
    {
	mico_throw(DialupManager::Syserr(l,"Returncode"));
    }
    corbalog.setprio(Syslog::Debug);
}


CORBA::Long
Queuemanager_impl::howmanyentries()
  throw(
    ::CORBA::SystemException,
    DialupManager::Syserr)

{
    CORBA::Long retval;
    int fd;
    char puffer[4096];
    char *p=puffer;
    int l;
    Process proc;
    
    if(queuemanager_debuglevel > 0)
	corbalog << iname << ".howmanyentries()" << endl;
    fd=proc.openreadpipe(0,queuelencmd);
    if(fd<0)
    {
	snprintf(puffer,1024,"openreadpipe:%s",strerror(errno));
	mico_throw(DialupManager::Syserr(errno,puffer));
    }
    while((l=read(fd,p,1024))>0 && p<puffer+4096)
    {
	p+=l;
    }
    proc.wait();
    if(l<0)
    {
	snprintf(puffer,1024,"read: %s",strerror(errno));
	mico_throw(DialupManager::Syserr(errno,puffer));
    }
    *p='\0';
    
    retval=atoi(puffer);
    return retval; 
}


void
Queuemanager_impl::listentries( CORBA::ULong howmany, DialupManager::EntrydescList*& list, DialupManager::QueueentryIterator_ptr& queit )
  throw(
    ::CORBA::SystemException,
    DialupManager::Syserr)
{
    if(queuemanager_debuglevel > 0)
	corbalog << iname << "listentries " << endl;
    QueueentryIterator_impl *mqueit = new QueueentryIterator_impl;
    if(mqueit->open(uid,mailqcmd)<0)
    {
	mico_throw(DialupManager::Syserr(errno,strerror(errno)));
    }
    if(howmany > 0)
    {
	mqueit->next_n(howmany,list);
    }
    else
    {
	list=NULL;
    }
    if(mqueit->eof())
    {
	mqueit->ende();
	delete mqueit;
	queit = DialupManager::QueueentryIterator::_nil();
    }
    else
    {
	queit = mqueit->_this();
    }
}


// Implementation for interface QueueentryIterator
int QueueentryIterator_impl::open(int uid,char *cmd)
{
    if((fd=proc.openreadpipe(uid,cmd))<0) return fd;
    lq.attach(fd);
    if(!lq)
    {
	corbalog.log(Syslog::Err,"openreadpide:Fehler beim lesen:%m");
	return -1;
    }
    return fd;
}

CORBA::Boolean
QueueentryIterator_impl::next_one( char*& entry )
  throw(
    ::CORBA::SystemException)

{
  CORBA::Boolean retval;

  if(lq.eof()) return FALSE;
  lq.getline(puffer,1024);
  entry = strdup(puffer);
  return TRUE; 
}


CORBA::Boolean
QueueentryIterator_impl::next_n( CORBA::ULong how_many, DialupManager::EntrydescList*& bl )
  throw(
    ::CORBA::SystemException)

{
  CORBA::Boolean retval=TRUE;
  CORBA::ULong i=0;
  char *z;
  if(how_many == 0) return FALSE;
  bl=new DialupManager::EntrydescList;
  bl->length(how_many);
  while(i<how_many && next_one(z))
  {
      (*bl)[i]=z;
      i++;
  }
  if(i<how_many) bl->length(i); 
  retval = (i>0);
  return retval; 
}


void
QueueentryIterator_impl::destroy()
  throw(
    ::CORBA::SystemException)

{
    extern CORBA::ORB_ptr orb;
    if(queuemanager_debuglevel > 0)
	corbalog << "Zerstre QueueentryIterator ... " << endl;
    ende();
    CORBA::Object_var obj = orb->resolve_initial_references ("POACurrent");
    PortableServer::Current_var pc = PortableServer::Current::_narrow (obj);
    PortableServer::POA_var poa = pc->get_POA ();
    PortableServer::ObjectId_var oid = pc->get_object_id ();
    poa->deactivate_object (oid.in());
}

/*------------------ sonstige Funktionen -----------------------*/
int Queuemanager_impl::lese(const char *datei)
{
  char puffer[1024];

  fstream beschr(datei,ios::in);
  if(!beschr)
  {
      corbalog.log(Syslog::Err,"%s kann nicht gelesen werden !",datei);
      return -1;
  }
 
  Devicelistentry **aktl = &ddlwurzel;

  while(beschr.getline(puffer,1024,'\n'))
  {
      if(puffer[0]=='#') continue;

      if(strncmp(puffer,"description=",12)==0)
      {
	  idesc=strdup(puffer+12);
      }
      else if(strncmp(puffer,"uid=",4)==0)
      {
	 char *ep;
	 uid=strtol(puffer+4,&ep,10);
      }
      else if(strncmp(puffer,"fetchcmd=",9)==0)
      {
	  fetchcmd = strdup(puffer+9);
      }
      else if(strncmp(puffer,"mailqcmd=",9)==0)
      {
	  mailqcmd= strdup(puffer+9);
      }  
      else if(strncmp(puffer,"queuelencmd=",12)==0)
      {                     
	  queuelencmd=strdup(puffer+12);
      }
      else if(strncmp(puffer,"dialdevice",10)==0)
      {
	  char *p=puffer+10;
	  while(*p==' '|| *p=='\n'||*p==':'||*p=='=') p++;
	  *aktl = new Devicelistentry();
	  try
	  {
	      (*aktl)->init(p);
	      aktl=&((*aktl)->nxt);
	  }
	  catch(::CORBA::SystemException_catch &ex)
	  {
	      corbalog << p << " nicht brauchbar: ";
	      ex._print(corbalog);
	      corbalog << endl;
	      *aktl=NULL;  
	  }
	  catch(...)
	  {
	      corbalog << p << " nicht brauchbar " << endl;
	      *aktl=NULL;
	  }
      }
      else
      {
	  corbalog.log(Syslog::Warning,"Unbekannt: %s",puffer);
      }
  }
  beschr.close();
}

extern CORBA::ORB_ptr orb;

void Devicelistentry::init(const char *name)
    throw(::CORBA::SystemException,DialupManager::Syserr)
{
    CORBA::Object_var obj= orb->string_to_object(name);
    if(orb->non_existent(obj))
	mico_throw(DialupManager::Syserr(-1,"Objekt existiert nicht"));
    dd = DialupManager::DialDevice::_narrow(obj);
    if(CORBA::is_nil(dd))
	mico_throw(DialupManager::Syserr(-1,"Objekt kein Dialdevice"));
}

void Queuemanager_impl::opendevices()
{
    if(queuemanager_debuglevel > 0)
	corbalog << "Kontrolliere verbundene Dialout-Devices... " << endl;
    for(Devicelistentry *akt=ddlwurzel;akt!=NULL;akt=akt->nxt)
    {
	try
	{
	    if(queuemanager_debuglevel > 0)
		corbalog << akt->dd->name() << endl;
	    akt->alter_dm = akt->dd->getdialmode();
	    if(akt->alter_dm != DialupManager::DialDevice::automatic)
	    {
		akt->dd->setdialmode(DialupManager::DialDevice::automatic);
	    }
	}
	catch(...)
	{
	    corbalog << "Hupsa !" << endl;
	}
    }
    if(queuemanager_debuglevel > 0)
	corbalog << "Alle sind auf auto ... " << endl;
}

void Queuemanager_impl::closedevices()
{
    if(queuemanager_debuglevel > 0)
	corbalog << "Setze verbundene Dialout-Devices zurck ... " << endl;
    for(Devicelistentry *akt=ddlwurzel;akt!=NULL;akt=akt->nxt)
    {
	try
	{
	    if(queuemanager_debuglevel > 0)
		corbalog << akt->dd->name() << endl;
	    if(akt->alter_dm != DialupManager::DialDevice::automatic)
	    {
		akt->dd->setdialmode(akt->alter_dm);
	    }
	}
	catch(...)
	{
	    corbalog << "Hupsa !" << endl;
	}
    }
    if(queuemanager_debuglevel > 0)
	corbalog << "Alle sind zurckgesetzt ... " << endl;
}


void Queuemanager_impl::testaus(ostream &ostr)
{
    ostr << "description=" << idesc << endl;
    ostr << "uid=" << uid << endl;
    ostr << "fetchcmd=" << fetchcmd << endl;
    ostr << "mailqcmd=" << mailqcmd << endl;
    ostr << "queuelencmd=" << queuelencmd << endl;
}





