#include<unistd.h>
#include "control_impl.h"
#ifndef LIBSTD
#include<string.h>
#endif

#ifdef LIBSTD
// Implementation for interface Control
Control_impl::Control_impl(const char *pname)
{
    iname=pname;
}
#else
Control_impl::Control_impl(const char *pname,Debugvar *_dvm,int _dvmsize)
{
   iname=pname;
   dvm = _dvm;
   dvmsize = _dvmsize-1;
   assert(dvm[dvmsize].name==NULL);
}
#endif

char *
Control_impl::name()
    throw(::CORBA::SystemException)
{
    return CORBA::string_dup(iname);
}

CORBA::Long 
Control_impl::getpid()
	throw(::CORBA::SystemException)
{
    return (CORBA::Long) ::getpid();
}

AppControl::Debugseq*
Control_impl::getdebugvars()
  throw(
    ::CORBA::SystemException)

{
#ifdef LIBSTD
  Debugvarmap::iterator it;
  AppControl::Debugseq* retval = new AppControl::Debugseq();
  retval->length(dvm.size());
  int i=0;
  for(it=dvm.begin();it!=dvm.end();++it)
  {
      (*retval)[i].facility = CORBA::string_dup((*it).first.data()); 
      (*retval)[i].level = *(it->second);
      i++;
  }
  return retval;
#else
  AppControl::Debugseq* retval = new AppControl::Debugseq();
  retval->length(dvmsize);
  int i=0;
  for(Debugvar *akt=dvm;akt->name!=NULL;akt++)
  {
      (*retval)[i].facility=CORBA::string_dup(akt->name);
      (*retval)[i].level = *(akt->pval);
      i++;
  }
  return retval;
#endif 
}

#ifndef LIBSTD
Debugvar *Control_impl::find(const char *name)
{
   Debugvar *akt=dvm;
   while(akt->name!=NULL && strcmp(name,akt->name)!=0) akt++;
   
   return ( akt->name!=NULL ? akt : (Debugvar *)NULL);
}
#endif

void
Control_impl::setdebugvars( const AppControl::Debugseq& ds )
  throw(
    ::CORBA::SystemException)

{
    for(int i=0;i<ds.length();i++)
    {
#ifdef STDLIB
	string facility(ds[i].facility);
	Debugvarmap::iterator it = dvm.find(facility);
	if(it!=dvm.end())
	{
	    *(it->second)=ds[i].level;
        }
#else
        Debugvar *akt;
	if((akt=find(ds[i].facility))!=NULL)
	{
	    *(akt->pval)=ds[i].level;
	}
#endif
    }
    // print_debugtab(cout);
}


void
Control_impl::exportnames(CosNaming::NamingContext_ptr nc )
  throw(
    ::CORBA::SystemException)

{
    mico_throw(::CORBA::NO_IMPLEMENT());
}

#ifdef LIBSTD
void Control_impl::init_debugtab()
{
}
#endif

void Control_impl::print_debugtab(ostream &str)
{
#ifdef LIBSTD
   for(Debugvarmap::iterator it=dvm.begin();it!=dvm.end();++it)
   {
      str << it->first << " " << *(it->second) << endl; 
   }
#else
   for(Debugvar *akt=dvm;akt->name != NULL; akt++)
   {
       str << akt->name << " " << *(akt->pval) << endl;
   }
#endif
}  






