#include<string.h>
#include<unistd.h>
#include<stdio.h>
#include<sys/time.h>
#include<sys/stat.h>
#include<fcntl.h>
#include<signal.h>
#include<iostream.h>
#include<fstream.h>
#include<mico/CosEventChannelAdmin.h>
#include<mico/CosNaming.h>
#include"corbalib.h"
#include"syslog.h"
#include<sys/errno.h>

#ifndef PREFIX
#define PREFIX "/local/corbammims-0.2/"
#endif
#define PIDFILE PREFIX "/run/fifo2ev.pid"

CosEventChannelAdmin::ProxyPushConsumer_var pc;
int verbose=0;
CORBA::ORB_var orb;

void sighandler (int sig)
{
  if (!CORBA::is_nil (orb))
  {
    orb->shutdown (TRUE);
  }
}

class PushSupplier_impl : virtual public CosEventComm::PushSupplier_skel
{
public:
    PushSupplier_impl() {}
    void disconnect_push_supplier();
};

void PushSupplier_impl::disconnect_push_supplier()
{
    exit(1);
}

/*------------ Ereignisbehandler ------------------------------*/
class PipeEreignisCallback: public CORBA::DispatcherCallback
{
    /* eigene Klasse ? */
    int offset;
    Syslog::Facility_var fac;
    Syslog::Prog_var prog;
    Syslog::Priority prio;
    CORBA::Long pid;
    CORBA::Long fd;
    char location[32];

    
    char puffer[4097];
public:
    PipeEreignisCallback(const char *path,const char *pfac,const char *pprog,
			 pid_t pidd,Syslog::Priority prio);
    int filed() { return fd; }
    void callback(CORBA::Dispatcher *, CORBA::Dispatcher::Event);
};

PipeEreignisCallback::PipeEreignisCallback(const char *path,const char *pfac,
					   const char *pprog,pid_t ppid,
					   Syslog::Priority pprio)
{
    struct stat statbuf;

    fac=pfac;
    prog=pprog;
    pid=ppid;
    prio=pprio;
    gethostname(location,32);

    /* ffne Datei */
    fd=open(path,O_RDONLY|O_NONBLOCK);
    /* NONBLOCK sonst wartet open auf schreiber */
    if(fd<0)
    {
	cerr << "Kann Datei " << path << " nicht ffnen: "
	     << strerror(errno) << endl;
	exit(1);
    }
    if(fstat(fd,&statbuf)<0) 
    {
	perror("stat");
	exit(1);
    }
    if(S_ISREG(statbuf.st_mode))
    {
	if(verbose)
	{
	    cerr << "Regulre Datei " << path
		 << " -> positioniere ans Ende " << endl;
	}
	lseek(fd,0,SEEK_END);
    }
}

Syslog::Priority str2prio(const char *str)
{
    static const char *prios[] = { "Alert","Crit","Err","Warning","Notice",
				   "Info","Debug",NULL };
    int r=0;
    int i = 0;
    while(prios[i]!=NULL && !(r=(strcasecmp(prios[i],str)==0))) i++;
    return (r ? Syslog::Notice : (Syslog::Priority)i);
};

void PipeEreignisCallback::callback(CORBA::Dispatcher *disp, CORBA::Dispatcher::Event ev)
{

    char puffer[4097];
    struct timeval tv;
    gettimeofday(&tv,NULL);
    CORBA::Any msg;
    Syslog::LogRecordUnix lr;
    int l;

    l=::read(fd,puffer,4096);
    
    if(l<=0)
    {
	if(l<0 && errno != EAGAIN) 
	{
	    perror("EingabezuEreignisCallback::read");
	}
	if(ev == CORBA::Dispatcher::Read)
	{
	    disp->remove(this,CORBA::Dispatcher::Read);
	}
	disp->tm_event(this,5000);
	if(verbose) cerr << "EOF, warte auf neues " << endl;
	return;
    }
    if(l>0 && ev == CORBA::Dispatcher::Timer)
    {
	if(verbose) cerr << "Hnge Event in Read-Queue ein " << endl;
	disp->rd_event(this,filed());
    }

    if(puffer[l-1]=='\n')
	puffer[l-1]='\0';
    else
	puffer[l]='\0';
    
    lr.facility = (const char *)fac;
    lr.prog     = (const char *)prog;
    lr.pid      = pid;
    lr.location = (const char *)location;
    lr.time     = tv.tv_sec;
    lr.priority = prio;
    lr.message  = (const char *)puffer;
    
    msg <<= lr;
    
    try
    {
	pc->push(msg);
    }
    catch(CORBA::SystemException_catch ex)
    {
	cerr << "bei push:";
	ex->_print(cerr);
	cerr << endl;
    }
}

void installenv()
{
   /* Signalhandler einrichten, Pidfile-schreiben */
    signal (SIGINT, sighandler);
    signal (SIGTERM, sighandler);
    ofstream pidstream(PIDFILE);
    if(!pidstream)
    {
	cerr << "Kann " << PIDFILE << " nicht schreiben " << endl;
	return;
    }
    pidstream << getpid() << endl;
}

main(int argc,char **argv)
{
    installenv();
    orb=CORBA::ORB_init(argc,argv,"mico-local-orb");
    CORBA::BOA_var boa=orb->BOA_init(argc,argv,"mico-local-boa");

    /*----- Initialisiere Corbalogger -----------------------*/
    CosNaming::Name syslogevname;
    CORBA::Object_var syslogevobj;
    CORBA::Object_var nsobj = orb->resolve_initial_references("NameService");
    if(CORBA::is_nil(nsobj))
    {
	cerr << "Hallo: Initial Service NameService not resolved " << endl;
	exit(1);
    }
    CosNaming::NamingContext_var nc =CosNaming::NamingContext::_narrow (nsobj);
    

    try
    {
	syslogevobj =nc->resolve(StringToName(syslogevname,"syslog"));
    }
    catch(...)
    {
	cerr << "Kann syslog nicht finden !! " << endl;
	exit(1);
    }
    CosEventChannelAdmin::EventChannel_var syslogev = CosEventChannelAdmin::EventChannel::_narrow(syslogevobj);
    assert(!CORBA::is_nil (syslogev));

    /*--------------- Einhngen als Pushsupplier ---------*/
    CosEventChannelAdmin::SupplierAdmin_var sa = syslogev->for_suppliers();
    PushSupplier_impl *psi = new PushSupplier_impl();
    CosEventComm::PushSupplier_var ps = psi->_this();
    pc = sa->obtain_push_consumer();
    pc->connect_push_supplier(ps);

    boa->impl_is_ready(CORBA::ImplementationDef::_nil());
   
    /*--------- Lese Dateien aus Eingabedatei ----------*/
    char **arg=argv+1;

    while(*arg!=NULL && **arg=='-')
    {
	if(strcmp(*arg,"-v")==0)
	{
	    verbose=1;
	}
	arg++;
    }
    if(*arg==NULL)
    {
	cerr << "Aufruf:" << argv[0] << " [-v] <beschreibungsdatei> " << endl;
	exit(1);
    }
    char *datei = *arg;
    if(verbose) cerr << "Lese datei " << datei << endl;

    char puffer[1024];
    char path[128];
    char facility[32];
    char prog[32];
    int  pid;
    char sprio[16];

    int zeile;
    PipeEreignisCallback *eecb;

    ifstream descr(datei,ios::in);
    if(!descr)
    {
	cerr << "Kann " << datei << " nicht ffnen " << endl;
	exit(1);
    }

    while(descr.getline(puffer,1024))
    {
      zeile++;
      if(puffer[0]=='#') continue;
      if(sscanf(puffer,"%s %s %s %d %s",path,facility,prog,&pid,sprio)!=5)
      {
	  cerr << "Syntaxfehler in Zeile " << zeile << endl;
	  continue;
      }
      if(verbose)
      {
	  cerr << "---- Zeile " << zeile << "---- " << endl;
	  cerr << "Path\t: " << path << '\n';
	  cerr << "Facility\t: " << facility << '\n';
	  cerr << "Progi\t: "    << prog << '\n';
	  cerr << "pid \t: "     << pid  << '\n';
	  cerr << "prio\t: "     << sprio << '\n';
      }
      eecb = new PipeEreignisCallback(path,facility,prog,
				      (pid!=-1 ? pid : getpid()),
				      str2prio(sprio));
      cerr << "Haenge " << prog  << " ein " << endl;
      orb->dispatcher()->rd_event(eecb,eecb->filed());
    }
    orb->run();
    CORBA::release(psi);
}







