#include<iostream.h>
#include<time.h>
#include<signal.h>
#include<sys/uio.h>
#include<mico/dynany.h>
#include<mico/CosEventChannelAdmin.h>
#include"pushconsumer_impl.h"

/*------- Globale Referenzen -----------------------*/
CORBA::ORB_var orb = CORBA::ORB::_nil();
DynamicAny::DynAnyFactory_var dynaf;
CosEventChannelAdmin::ProxyPushSupplier_var supplier;


// Implementation for interface PushConsumer

void
PushConsumer_impl::push( const CORBA::Any& any)
  throw(
    ::CORBA::SystemException,
    CosEventComm::Disconnected)

{
    struct iovec iov[10];

    char timestr[32];
    CORBA::String_var facility;
    CORBA::String_var prog;
    CORBA::String_var message;
    char pidstr[16];
    
    for(int i=0;i<10;i++)
    {
	iov[i].iov_len=0;
	iov[i].iov_base = (void *)"";
    }
    assert(!CORBA::is_nil(dynaf));
    DynamicAny::DynAny_var dyna = dynaf->create_dyn_any(any);
    CORBA::TypeCode_ptr tc = dyna->type();
    if(tc->kind() == CORBA::tk_struct)
    {
      DynamicAny::DynStruct_var dynstr = DynamicAny::DynStruct::_narrow(dyna);
      CORBA::String_var mn;
      DynamicAny::DynAny_var comp;
      try
      {
	do
	{
	  mn = dynstr->current_member_name();
	  comp = dynstr->current_component();
	  if(strcmp(mn,"time")==0)
	  {
	    time_t t = (time_t)comp->get_long();
	    iov[0].iov_len =strftime(timestr,32,"%d.%m.%Y %H:%M:%S ",localtime(&t));
	    iov[0].iov_base=timestr;
	  }
	  else if(strcmp(mn,"facility")==0)
	  {
	      facility = comp->get_string();
	      iov[1].iov_len =strlen(facility);
	      iov[1].iov_base=facility;
	      iov[2].iov_len =1;
	      iov[2].iov_base=(void *)" ";
	  }
	  else if(strcmp(mn,"prog")==0)
	  {
	      prog=comp->get_string();
	      iov[3].iov_len =strlen(prog);
	      iov[3].iov_base=prog;
	  }
	  else if(strcmp(mn,"pid")==0)
	  {
	      CORBA::Long pid = comp->get_long();
	      snprintf(pidstr,16,"[%d] ",pid);
	      iov[4].iov_len =strlen(pidstr);
	      iov[4].iov_base=pidstr;
	  }
	  else if(strcmp(mn,"message")==0)
	  {
	      message=comp->get_string();
	      iov[5].iov_len =strlen(message);
	      iov[5].iov_base=message;
	      iov[6].iov_len = 1;
              iov[6].iov_base=(void *)"\n";
	  }
	} while(dynstr->next());
      }
      catch(DynamicAny::DynAny::TypeMismatch_catch &ex)
      {
	  cerr << "TypeMismatch" << endl;
      }
      catch(DynamicAny::DynAny::InvalidValue_catch &ex)
      {
	  cerr << "InvalidValue" << endl;
      }
      if(writev(1,iov,7)<0)
      {
	  perror("INTERNAL ERROR: writev");
      }
    }
    else
    {
	cerr << "INTERNAL ERROR: kein Struct " << endl;
    }
    dyna->destroy();
}


void
PushConsumer_impl::disconnect_push_consumer()
  throw(
    ::CORBA::SystemException)

{
  cerr << "INTERN: Ende der Verbindung ... " << endl;
  ::orb->shutdown(TRUE);
}


void sighandler(int sig)
{
  if(!CORBA::is_nil (orb))
  {
    if(!CORBA::is_nil(supplier))
    {
	supplier->disconnect_push_supplier();
    }
    orb->shutdown(TRUE);
  }
}

void install_sighandler(int sig)
{
    struct sigaction action;
    action.sa_handler=sighandler;
    sigemptyset(&(action.sa_mask));
    action.sa_flags=0;
    
    sigaction(sig,&action,NULL);
}

void main(int argc,char **argv)
{
    install_sighandler(SIGINT);
    install_sighandler(SIGTERM);
    orb=CORBA::ORB_init(argc,argv,"mico-local-orb");
    CORBA::BOA_var boa = orb->BOA_init(argc,argv,"mico-local-boa");
    if(argc < 2)
    {
	cerr << "Aufruf: " << argv[0] << " objref " << endl;
	exit(1);
    }
    CORBA::Object_var dynafo = orb->resolve_initial_references("DynAnyFactory");
    dynaf = DynamicAny::DynAnyFactory::_narrow(dynafo);

    const char *url=argv[1];
    PushConsumer_impl *pci = new PushConsumer_impl();
    CosEventComm::PushConsumer_var mein_pc = pci->_this();

    CORBA::Object_var evchobj = orb->string_to_object(url);
    CosEventChannelAdmin::EventChannel_var evch = CosEventChannelAdmin::EventChannel::_narrow(evchobj);
    
    if(CORBA::is_nil(evch))
    {
	cerr << url << " ist kein CosEventChannelAdmin::EventChannel " << endl;
	exit(1);
    }
    CosEventChannelAdmin::ConsumerAdmin_var admin=evch->for_consumers();
    supplier=admin->obtain_push_supplier();
    supplier->connect_push_consumer(mein_pc);
    
    boa->impl_is_ready(CORBA::ImplementationDef::_nil());
    orb->run();
}






